#dir_listing_pool{
	display: flex;
	flex-direction: column;
}
.list_entry{
	display: flex;
	position: relative;
	padding: 2px;
	padding-left: calc(20px + 5px);
	margin: 1px;
	margin-top: 5px;
	align-items: center;
	font-size: 20px;
	margin-left: 25px;
}
.list_entry:hover{
	background: rgba(0, 0, 0, 0.1);
}
.actual_link{
	width: 100%;
	height: 100%;
}
.list_entry::before{
	content: '';
	position: absolute;
	left: 0px;
	height: 100%;
	width: 20px;
	background-position: center;
	background-repeat: no-repeat;
	background-size: contain;
}
.list_entry[dir_entry]::before{
	background-image: url('data:image/svg+xml;base64,$$folder_icon_b64');
}
.list_entry[file_entry]::before{
	background-image: url('data:image/svg+xml;base64,$$file_icon_b64');
}

.dl_button{
	position: absolute;
	left: -25px;
	height: 100%;
	width: 20px;
	background-image: url('data:image/svg+xml;base64,$$dl_icon_b64');
	background-size: contain;
	background-position: center;
	background-repeat: no-repeat;
}

.list_entry[dir_entry] .dl_button{
	display: none;
}

/* Have better ideas? Comment on github (PLEASE) */
@media only screen and (hover: none) and (pointer: coarse){
	.list_entry{
		font-size: 7.5vw !important;
		padding: 7px;
		padding-left: calc(7.5vw + 5px);
	}
	.list_entry::before{
		width: 7.5vw;
	}
}